/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qpainter.h>
#include "ShisenSho.h"
#include "Board.h"
#include <stdlib.h>

ShisenSho::ShisenSho(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f)
{
    /* initialize random number generator */
    srand(::time(0));

    /* set windows style and caption */
    setCaption(tr("Shisen-Sho"));

    /* create the play field as the central widget */
    m_board = new Board(this);
    setCentralWidget(m_board);
    m_board->show();
    connect(m_board, SIGNAL(changed()), this, SLOT(enableItems()));
    connect(m_board, SIGNAL(endOfGame()), this, SLOT(gameOver()));

    /* get a timer that updates the board */
    QTimer *timer = new QTimer(this);
    timer->start(1000);
    connect(timer, SIGNAL(timeout()), this, SLOT(updateScore()));

    /* create the config/speed submenu */
    m_speedMenu = new QPopupMenu(0, "speed_menu");
    m_speedMenu->setCheckable(true);
    connect(m_speedMenu, SIGNAL(activated(int)), this, SLOT(updateSpeed(int)));
    m_speedMenu->insertItem(tr("Very slow"), 125);
    m_speedMenu->insertItem(tr("Slow"), 250);
    m_speedMenu->insertItem(tr("Medium"), 500);
    m_speedMenu->insertItem(tr("Fast"), 750);
    m_speedMenu->insertItem(tr("Very fast"), 1000);

    /* create the config/size submenu */
    m_sizeMenu = new QPopupMenu(0, "size_menu");
    m_sizeMenu->setCheckable(true);
    connect(m_sizeMenu, SIGNAL(activated(int)), this, SLOT(updateSize(int)));
    m_sizeMenu->insertItem(tr("8x6"), 0);
    m_sizeMenu->insertItem(tr("8x8"), 1);
    m_sizeMenu->insertItem(tr("10x6"), 2);
    m_sizeMenu->insertItem(tr("10x8"), 3);

    /* create the config/difficulty submenu */
    m_iqMenu = new QPopupMenu(0, "iq_menu");
    m_iqMenu->setCheckable(true);
    connect(m_iqMenu, SIGNAL(activated(int)), this, SLOT(updateIQ(int)));
    m_iqMenu->insertItem(tr("Easy"), 1);
    m_iqMenu->insertItem(tr("Normal"), 5);
    m_iqMenu->insertItem(tr("Hard"), 9);
    
    /* create the config/gravity submenu */
    m_gravityMenu = new QPopupMenu(0, "gravity_menu");
    m_gravityMenu->setCheckable(true);
    connect(m_gravityMenu, SIGNAL(activated(int)), this, SLOT(updateGravity(int)));
    m_gravityMenu->insertItem(tr("No"), 0);
    m_gravityMenu->insertItem(tr("Yes"), 1);

    /* create the game submenu */
    m_gameMenu = new QPopupMenu(0, "game_menu");
    m_gameMenu->insertItem(tr("New"), this, SLOT(start()));
    m_gameMenu->insertSeparator();
    m_gameMenu->insertItem(tr("Hint"), this, SLOT(hint()));
    m_gameMenu->insertItem(tr("Undo"), this, SLOT(undo()), 0, 0x0100);
    m_gameMenu->insertItem(tr("Redo"), this, SLOT(redo()), 0, 0x0101);
    m_gameMenu->insertSeparator();
    m_gameMenu->insertItem(tr("Close"), this, SLOT(close()));

    /* create the config submenu */
    QPopupMenu *configMenu = new QPopupMenu(0, "config_menu");
    configMenu->insertItem(tr("Speed"), m_speedMenu);
    configMenu->insertItem(tr("Size"), m_sizeMenu);
    configMenu->insertItem(tr("Difficulty"), m_iqMenu);
    configMenu->insertItem(tr("Gravity"), m_gravityMenu);

    /* create the menu */
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem(tr("Game"), m_gameMenu);
    menu->insertItem(tr("Configuration"), configMenu);

    /* read the configuration file */
    readConfig();

    /* check the menu entries of game size and game iq submenu */
    m_speedMenu->setItemChecked(m_speed, true);
    m_sizeMenu->setItemChecked(m_size, true);
    m_iqMenu->setItemChecked(m_iq, true);
    m_gravityMenu->setItemChecked(m_gravity, true);
    enableItems();

    /* at last, set up the board */
    m_board->setSolvableFlag(true);
    m_board->setGravityFlag(m_gravity);
    m_board->setShuffle(m_iq);
    m_board->setDelay(m_speed);
    switch (m_size) {
    case 0: m_board->setSize(8, 6); break;
    case 1: m_board->setSize(8, 8); break;
    case 2: m_board->setSize(10, 6); break;
    case 3: m_board->setSize(10, 8); break;
    }

    /* start a game automatically */
    start();
}

ShisenSho::~ShisenSho()
{
    writeConfig();
}

void ShisenSho::readConfig(void)
{
    Config cfg("ShisenSho");

    cfg.setGroup("Settings");

    int speed = cfg.readNumEntry("Speed", 500);
    if (speed <= 125) {
	m_speed = 125;
    } else if (speed <= 250) {
	m_speed = 250;
    } else if (speed <= 500) {
	m_speed = 500;
    } else if (speed <= 750) {
	m_speed = 750;
    } else {
	m_speed = 1000;
    }

    int size = cfg.readNumEntry("Size", 3);
    if (size <= 0) {
	m_size = 0;
    } else if (size >= 3) {
	m_size = 3;
    } else {
	m_size = size;
    }

    int iq = cfg.readNumEntry("IQ", 5);
    if (iq <= 1) {
	m_iq = 1;
    } else if (iq <= 5) {
	m_iq = 5;
    } else {
	m_iq = 9;
    }

    int gravity = cfg.readNumEntry("Gravity", 0);
    if (gravity <= 0) {
	m_gravity = 0;
    } else {
	m_gravity = 1;
    }
}

void ShisenSho::writeConfig(void)
{
    Config cfg("ShisenSho");

    cfg.setGroup("Settings");
    cfg.writeEntry("Speed", m_speed);
    cfg.writeEntry("Size", m_size);
    cfg.writeEntry("IQ", m_iq);
    cfg.writeEntry("Gravity", m_gravity);
}

void ShisenSho::updateSpeed(int speed)
{
    m_speedMenu->setItemChecked(m_speed, false);
    m_speed = speed;
    m_speedMenu->setItemChecked(m_speed, true);
    writeConfig();
    if (!m_board->canUndo()) {
	m_board->setDelay(speed);
    } else {
	m_board->setDelay(speed);
	start();
    }
}

void ShisenSho::updateSize(int size)
{
    m_sizeMenu->setItemChecked(m_size, false);
    m_size = size;
    m_sizeMenu->setItemChecked(m_size, true);
    writeConfig();
    switch (size) {
    case 0: m_board->setSize(8, 6); break;
    case 1: m_board->setSize(8, 8); break;
    case 2: m_board->setSize(10, 6); break;
    case 3: m_board->setSize(10, 8); break;
    }
    start();
}

void ShisenSho::updateIQ(int iq)
{
    m_iqMenu->setItemChecked(m_iq, false);
    m_iq = iq;
    m_iqMenu->setItemChecked(m_iq, true);
    writeConfig();
    if (!m_board->canUndo()) {
	m_board->setShuffle(iq);
    } else {
	m_board->setShuffle(iq);
	start();
    }
}

void ShisenSho::updateGravity(int gravity)
{
    m_gravityMenu->setItemChecked(m_gravity, false);
    m_gravity = gravity;
    m_gravityMenu->setItemChecked(m_gravity, true);
    writeConfig();
    if (!m_board->canUndo()) {
	m_board->setGravityFlag(gravity);
    } else {
	m_board->setGravityFlag(gravity);
	start();
    }
}

void ShisenSho::updateScore(void)
{
    m_board->repaint(0, 0, width(), 22, false);
}

void ShisenSho::start(void)
{
    m_board->setSolvableFlag(true);
    m_board->setCheated(false);
    m_board->newGame();
    enableItems();
}

void ShisenSho::hint(void)
{
    qApp->processEvents();
    m_board->getHint();
    m_board->setCheated(true);
    enableItems();
}

void ShisenSho::undo(void)
{
    if (m_board->canUndo()) {
	m_board->undo();
	m_board->setCheated(true);
	enableItems();
    }
}

void ShisenSho::redo(void)
{
    if(m_board->canRedo()) {
	m_board->redo();
	enableItems();
    }
}

void ShisenSho::enableItems(void)
{
    m_gameMenu->setItemEnabled(0x100, m_board->canUndo());
    m_gameMenu->setItemEnabled(0x101, m_board->canRedo());
}

void ShisenSho::gameOver(void)
{
    if (m_board->tilesLeft()) {
	QMessageBox::information(this, tr("Shisen-Sho"),
				 tr("No more moves\npossible\nYou loose"));
    } else {
	int seconds = m_board->getTimeForGame();
	QString s;
	s.sprintf("%02d:%02d:%02d", seconds / 3600, (seconds / 60) % 60, seconds % 60);
	QMessageBox::information(this, tr("Shisen-Sho"),
				 tr("Congratulations!\nYou made it in\n%1").arg(s));
    }
    start();
}

